﻿using Bharabas.Data;
using Bharabas.Models;
using Bharabas.Scraping;
using Microsoft.IdentityModel.Tokens;
using System.ServiceModel.Syndication;
using System.Xml;

namespace Bharabas.Scraping
{
	public class ScrapingRSS
	{
		private readonly IHttpClientFactory _httpClientFactory;
		private readonly ApplicationDbContext _db;
		readonly string[] allowedLanguages = new string[] { "en", "in", "id" };

		public ScrapingRSS(ApplicationDbContext db, IHttpClientFactory httpClientFactory)
		{
			_httpClientFactory = httpClientFactory;
			_db = db;
		}

		public async Task scrapeRSSAsync(string rssUrl)
		{
			try
			{
				SyndicationFeed feed = await getSyndicationFeed(rssUrl);
				if(!isAllowedFeed(feed))
				{
					return;
				}
				(string podcastName, string defaultAuthor, string defaultImageUrl, HashSet<int> categoriesId) = getPodcastInfo(feed);
				foreach(SyndicationItem eps in feed.Items)
				{
					Audio newAudio = getEpsAudio(eps, podcastName, defaultAuthor, defaultImageUrl);
					if(await isValidUrl(newAudio.platformUrl))
					{
						if(!FilterInappropriateContent.isContainBadWords(newAudio.title))
						{
							processAudioToDb(newAudio, categoriesId);
						}
					}
				}
			} catch(Exception ex) { Console.WriteLine(ex); }
		}

		public async Task<SyndicationFeed> getSyndicationFeed(string rssUrl)
		{
			if (_httpClientFactory == null)
			{
				throw new Exception("factory is null");
			}
			var client = _httpClientFactory.CreateClient();
			try
			{
				Stream stream = await client.GetStreamAsync(rssUrl);
				XmlReader reader = XmlReader.Create(stream);
				SyndicationFeed feed = SyndicationFeed.Load(reader);
				return feed;
			} catch (Exception) { throw; }
		}
		
		public bool isAllowedFeed(SyndicationFeed feed)
		{
			try
			{
				string isExplicit = feed.ElementExtensions.ReadElementExtensions<XmlElement>("explicit", "http://www.itunes.com/dtds/podcast-1.0.dtd")[0].InnerText;
				if (isExplicit == "Yes")
				{
					return false;
				}
				else
				{
					Console.WriteLine("isExplicit: " + isExplicit);
				}
			}
			catch (Exception) { throw; }
			try
			{
				string lang = feed.Language;
				bool flag = false;
				Console.WriteLine("lang: " + lang);
				foreach (var allowedLang in allowedLanguages)
				{
					if (lang.StartsWith(allowedLang))
					{
						flag = true;
					}
				}
				return flag;
			}
			catch (Exception) { throw; }
		}

		public (string podcastName, string defaultAuthor, string defaultImageUrl, HashSet<int> categoriesId) getPodcastInfo(SyndicationFeed feed)
		{
			string podcastName = feed.Title.Text;
			string defaultAuthor = string.Empty;
			try
			{
				defaultAuthor = feed.ElementExtensions.ReadElementExtensions<XmlElement>("author", "http://www.itunes.com/dtds/podcast-1.0.dtd")[0].InnerText;
			}
			catch (Exception ex) { Console.WriteLine(ex); }
			string defaultImageUrl = string.Empty;
			try
			{
				defaultImageUrl = feed.ElementExtensions.ReadElementExtensions<XmlElement>("image", "http://www.itunes.com/dtds/podcast-1.0.dtd")[0].GetAttribute("href");
			}
			catch (Exception ex) { Console.WriteLine(ex); }
			HashSet<int> categoriesId = new HashSet<int>();
			try
			{
				var podcastCategories = feed.ElementExtensions.ReadElementExtensions<XmlElement>("category", "http://www.itunes.com/dtds/podcast-1.0.dtd");
				foreach (var podcastCategory in podcastCategories)
				{
					int categoryId = Util.findAndUpdateCategoryID(_db, podcastCategory.GetAttribute("text"));
					categoriesId.Add(categoryId);
				}
			}
			catch (Exception ex) { Console.WriteLine(ex); }
			return (podcastName, defaultAuthor, defaultImageUrl, categoriesId);
		}

		public Audio getEpsAudio(SyndicationItem eps, string podcastName, string defaultAuthor, string defaultImageUrl)
		{
			var newAudio = new Audio();
			var fullTitle = podcastName + " / " + eps.Title.Text;
			newAudio.title = Util.ConvertToPlainText(fullTitle).Replace("\r", "").Replace("\n", "").Trim();
			newAudio.description = eps.Summary == null ? string.Empty : Util.ConvertToPlainText(eps.Summary.Text).Replace("\r", "").Replace("\n", "").Trim();
			newAudio.url = string.Empty;
			//newAudio.platformUrl = epsUrl;
			int flag = 2;
			foreach (var link in eps.Links)
			{
				if (link.RelationshipType == "enclosure")
				{
					newAudio.url = link.Uri.ToString();
					flag--;
					if (flag == 0) break;
				}
				else
				{
					newAudio.platformUrl = link.Uri.ToString();
					flag--;
					if (flag == 0) break;
				}
			}
			if (newAudio.url == string.Empty || Util.isAudioPathExist(_db, newAudio.url))
			{
				throw new Exception("Audio Url not exist or already in database");
			}
			newAudio.platform = getPlatformFromUrl(newAudio.platformUrl);
			try
			{
				newAudio.imageUrl = eps.ElementExtensions.ReadElementExtensions<XmlElement>("image", "http://www.itunes.com/dtds/podcast-1.0.dtd")[0].GetAttribute("href");
			}
			catch (Exception ex)
			{
				newAudio.imageUrl = defaultImageUrl;
			}
			try
			{
				newAudio.contentCreator = eps.ElementExtensions.ReadElementExtensions<XmlElement>("Author", "http://www.itunes.com/dtds/podcast-1.0.dtd")[0].InnerText;
			}
			catch (Exception ex)
			{
				newAudio.contentCreator = defaultAuthor;
			}
			try
			{
				string durationText = eps.ElementExtensions.ReadElementExtensions<XmlElement>("duration", "http://www.itunes.com/dtds/podcast-1.0.dtd")[0].InnerText;
				if (durationText.Contains(':'))
				{
					newAudio.duration = Util.toSecond(durationText);
				}
				else
				{
					newAudio.duration = Convert.ToInt32(durationText);
				}
			}
			catch (Exception ex) { Console.WriteLine(ex); }
			try
			{
				DateTimeOffset pubDateTime = eps.PublishDate.ToLocalTime();
				newAudio.dateUploaded = DateOnly.FromDateTime(pubDateTime.DateTime);
			}
			catch (Exception ex) { Console.WriteLine(ex); }
			//Console.WriteLine(newAudio.title);
			//Console.WriteLine(newAudio.description);
			//Console.WriteLine(newAudio.url);
			//Console.WriteLine(newAudio.imageUrl);
			//Console.WriteLine(newAudio.contentCreator);
			//Console.WriteLine(newAudio.platform);
			//Console.WriteLine(newAudio.platformUrl);
			////duration, dateuploaded
			//Console.WriteLine(newAudio.duration);
			//Console.WriteLine(newAudio.dateUploaded.ToString());
			return newAudio;
		}

		public string getPlatformFromUrl(string platformUrl)
		{
			string res = string.Empty;
			var uri = new Uri(platformUrl);
			res = uri.Host;
			if(res.Contains("youtu.be"))
			{
				res = "YouTube";
				return res;
			} else if(res.Contains("podcasters.spotify"))
			{
				res = "Anchor";
				return res;
			} else if (res.Contains("podcasts.google"))
			{
				res = "Google Podcast";
				return res;
			} else if (res.Contains("listennotes"))
			{
				res = "ListenNotes";
				return res;
			} else if(res.Contains("spotify"))
			{
				res = "Spotify";
				return res;
			} 
			if(res.StartsWith("www."))
			{
				res = res.Remove(0, 4);
			}
			for(int i=res.Length-1;i>=0;i--)
			{
				if (res[i]=='.')
				{
					res = res.Substring(0, i);
					break;
				}
			}
			return res;
		}

		public async Task<bool> isValidUrl(string platformUrl)
		{
			if(platformUrl==null || platformUrl==string.Empty) return false;
			var client = _httpClientFactory.CreateClient();
			try
			{
				HttpResponseMessage response = await client.GetAsync(platformUrl);
				if (response.IsSuccessStatusCode)
				{
					return true;
				}
				else
				{
					Console.WriteLine($"Error: {response.StatusCode}" + platformUrl);
					return false;
				}
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex);
				return false;
			}
		}

		public void processAudioToDb(Audio newAudio, HashSet<int> categoriesId)
		{
			_db.AudioList.Add(newAudio);
			_db.SaveChanges();

			var epsId = _db.AudioList.Where(u => u.url == newAudio.url).Select(u => u.audioId).ToList()[0];

			// AudioCategory
			foreach (var categoryId in categoriesId)
			{
				//Console.WriteLine(categoryId);
				var newAudioCategoryRelation = new AudioCategoryRelation();
				newAudioCategoryRelation.audioId = epsId;
				newAudioCategoryRelation.categoryId = categoryId;
				_db.AudioCategoryRelation.Add(newAudioCategoryRelation);
			}
			_db.SaveChanges();
		}
	}
}
